{
 "cells": [
  {
   "cell_type": "code",
   "execution_count": 1,
   "metadata": {
    "collapsed": false
   },
   "outputs": [],
   "source": [
    "import vcsn"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 2,
   "metadata": {
    "collapsed": false
   },
   "outputs": [],
   "source": [
    "from IPython.display import display\n",
    "import ipywidgets as widgets\n",
    "import traitlets\n",
    "\n",
    "class Automaton(widgets.DOMWidget):\n",
    "    # the name of the Backbone.View subclass to be used\n",
    "    _view_name = traitlets.Unicode('AutomatonView').tag(sync=True)\n",
    "    _view_module = traitlets.Unicode('AutomatonD3Widget').tag(sync=True)\n",
    "    states = traitlets.List().tag(sync=True)\n",
    "    transitions = traitlets.List().tag(sync=True)\n",
    "    lastStateId = traitlets.CInt().tag(sync=True)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 3,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "data": {
      "application/javascript": [
       "require.config({paths: {d3: \"http://d3js.org/d3.v3.min\"}});"
      ],
      "text/plain": [
       "<IPython.core.display.Javascript object>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "%%javascript\n",
    "require.config({paths: {d3: \"http://d3js.org/d3.v3.min\"}});"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 4,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "data": {
      "application/javascript": [
       "require.undef('AutomatonD3Widget');\n",
       "\n",
       "define('AutomatonD3Widget',\n",
       "       ['jupyter-js-widgets', 'd3'],\n",
       "       function(widgets, d3) {\n",
       "    var AutomatonView = widgets.DOMWidgetView.extend({\n",
       "\n",
       "        // RENDER THE VIEW\n",
       "        render: function(){\n",
       "            // Here we define variables with 'this' to call them from everywhere\n",
       "\n",
       "            // Editable text for labels\n",
       "            this.svg_for_frame = d3.select(this.el).append('svg')\n",
       "                .attr('width', 960)\n",
       "                .attr('height', 100)\n",
       "            this.frame_title = this.svg_for_frame.append('svg:text')\n",
       "                .text('Label: ')\n",
       "                .attr('x', 25)\n",
       "                .attr('y', 25)\n",
       "            this.frame = this.svg_for_frame.append('foreignObject')\n",
       "                .attr('x', 85)\n",
       "                .attr('y', 10)\n",
       "                .attr(\"width\", 800)\n",
       "                .attr(\"height\", 50)\n",
       "                .append(\"xhtml:form\")\n",
       "                .attr('class', 'frame')\n",
       "                .append('input')\n",
       "\n",
       "            // Attributs of the SVG\n",
       "            this.width =960, this.height = 500;\n",
       "\n",
       "            // Render the svg\n",
       "            this.svg = d3.select(this.el).append('svg')\n",
       "                .attr('width', this.width)\n",
       "                .attr('height', this.height)\n",
       "                .style('userSelect' , \"none\")\n",
       "                .style('webkiUserSelect' , \"none\")\n",
       "                .style('MozUserSelect' , \"none\")\n",
       "                .style('oUserSelect', 'none')\n",
       "                .style('msUserSelect', 'none')\n",
       "                .style('draggable', 'false')\n",
       "                .classed('m', false)\n",
       "                .style('cursor', 'crosshair')\n",
       "\n",
       "            // Initialize datas\n",
       "            this.states = this.model.get('states');\n",
       "            var trans = this.model.get('transitions');\n",
       "            this.transitions = this.completeTransitions(trans);\n",
       "            this.lastStateId = this.states[this.states.length - 1].id;\n",
       "\n",
       "            // Define here to avoid recreation in every update\n",
       "            this.path = this.svg.append('svg:g').selectAll('g');\n",
       "            this.circle = this.svg.append('svg:g').selectAll('g');\n",
       "\n",
       "            // Lines displayed when draggind new states\n",
       "            this.drag_line = this.svg.append('svg:path')\n",
       "                .style('fill', 'none')\n",
       "                .style('stroke', '#000')\n",
       "                .style('stroke-width', '1.50px')\n",
       "                .attr('class', 'transition dragline hidden')\n",
       "                .attr('d', 'M0,0L0,0');\n",
       "\n",
       "            // Arrow for transitions\n",
       "            this.svg.append('svg:defs').append('svg:marker')\n",
       "                .attr('id', 'end-arrow')\n",
       "                .attr('viewBox', '-3 -10 20 20')\n",
       "                .attr('refX', 2)\n",
       "                .attr('markerWidth', 6)\n",
       "                .attr('markerHeight', 6)\n",
       "                .attr('orient', 'auto')\n",
       "              .append('svg:path')\n",
       "                .attr('d', 'M-3,12 l22,-12 l-22,-12 l7,12 Z')\n",
       "                .attr('fill', '#000');\n",
       "\n",
       "            // Closure\n",
       "            var that = this;\n",
       "\n",
       "            // Catch source and target with their ID\n",
       "            for(var c = 0; c <= this.transitions.length -1; c++)\n",
       "                {\n",
       "                    this.transitions[c].source = this.idToState(this.transitions[c].source);\n",
       "                    this.transitions[c].target = this.idToState(this.transitions[c].target);\n",
       "                }\n",
       "\n",
       "            // Force layout for physical simulation\n",
       "            this.force = d3.layout.force()\n",
       "                .nodes(this.states)\n",
       "                .links(this.transitions)\n",
       "                .size([this.width, this.height])\n",
       "                .linkDistance(function(d) {\n",
       "                    if(that.isShort(d)) {return 75;}\n",
       "                    else { return 175;}\n",
       "                })\n",
       "                .charge(function(d) {\n",
       "                    if(that.isHidden(d)) {return -100;}\n",
       "                    else {return -750;}\n",
       "                })\n",
       "                .on('tick', function(d,i){ that.stepForce(this, d, i);})\n",
       "\n",
       "            // Listen for mouse and keyboard events on the whole document\n",
       "            this.svg.on('mousedown', function(d,i){ that.mousedown(this, d, i);})\n",
       "                .on('mousemove', function(d,i){ that.mousemove(this, d, i);})\n",
       "                .on('mouseup', function(d,i){ that.mouseup(this, d, i);})\n",
       "            // Active keyboard action and disable notebook shortcuts when we are\n",
       "            // over the widget\n",
       "                .on(\"mouseover\", function(d,i) {\n",
       "                    // Disable some notebook key shortcuts\n",
       "                    var cmd = IPython.keyboard_manager.command_shortcuts;\n",
       "                    cmd.remove_shortcut('i');\n",
       "                    cmd.remove_shortcut('l');\n",
       "                    cmd.remove_shortcut('f');\n",
       "                    cmd.remove_shortcut('m');\n",
       "                    cmd.remove_shortcut('s');\n",
       "\n",
       "                    d3.select(window).on('keydown', function(d,i){\n",
       "                        that.keydown(this, d, i);})\n",
       "                        .on('keyup',  function(d,i){\n",
       "                            that.keyup(this, d, i);});\n",
       "                })\n",
       "            // if we leave the widget area, then disable keyboard action and\n",
       "            // reactive notebook shortcuts\n",
       "                .on(\"mouseout\", function(d,i) {\n",
       "                    // reset selections if we leave widget\n",
       "                    // that.selected_transition = null;\n",
       "                    // that.selected_state = null;\n",
       "                    // that.mousedown_state = null;\n",
       "                    // that.mouseup_state = null;\n",
       "                    // that.mousedown_transition = null;\n",
       "                    var cmd = IPython.keyboard_manager.command_shortcuts;\n",
       "                        cmd.add_shortcut('i', {\n",
       "                            help    : 'interrupt kernel (press twice)',\n",
       "                            help_index : 'ha',\n",
       "                            count: 2,\n",
       "                            handler : function (event) {\n",
       "                                IPython.notebook.kernel.interrupt();\n",
       "                                return false;\n",
       "                            }\n",
       "                        });\n",
       "                        cmd.add_shortcut('l', {\n",
       "                            help    : 'toggle line numbers',\n",
       "                            help_index : 'ga',\n",
       "                            handler : function (event) {\n",
       "                                IPython.notebook.cell_toggle_line_numbers();\n",
       "                                return false;\n",
       "                            }\n",
       "                        });\n",
       "                        cmd.add_shortcut('m', {\n",
       "                            help    : 'to markdown',\n",
       "                            help_index : 'cb',\n",
       "                            handler : function (event) {\n",
       "                                IPython.notebook.to_markdown();\n",
       "                                return false;\n",
       "                            }\n",
       "                        });\n",
       "                        cmd.add_shortcut('s', {\n",
       "                            help    : 'save notebook',\n",
       "                            help_index : 'fa',\n",
       "                            handler : function (event) {\n",
       "                                IPython.notebook.save_checkpoint();\n",
       "                                return false;\n",
       "                            }\n",
       "                        });\n",
       "                    d3.select(window).on('keydown', function() {\n",
       "                        return null})\n",
       "                        .on('keyup', function() {return null;});\n",
       "                });\n",
       "\n",
       "            this.hackUpdate();\n",
       "        },\n",
       "\n",
       "        hackUpdate: function(){\n",
       "            // Little hack to update model without touch the view\n",
       "            // Launched at the begginning avoid conversion error\n",
       "            // Between VCSN and D3\n",
       "\n",
       "            var old_transitions = this.transitions\n",
       "            var transition = {'source': '0', 'target':'0', 'label': ''}\n",
       "            var new_transitions = this.model.get('transitions').slice();\n",
       "            new_transitions.push(transition);\n",
       "            this.model.set('transitions', new_transitions);\n",
       "            this.touch();\n",
       "\n",
       "            this.model.set('transitions', old_transitions);\n",
       "            this.touch();\n",
       "\n",
       "            this.update();\n",
       "        },\n",
       "\n",
       "        // SET POSITIONS OF EVERY ELEMENTS\n",
       "        stepForce: function(doc, da, i) {\n",
       "\n",
       "            var that = this;\n",
       "            // Draw transitions with proper padding from state centers\n",
       "            this.path.selectAll('path').attr('d', function(d) {\n",
       "\n",
       "                var deltaX = d.target.x - d.source.x ,\n",
       "                deltaY = d.target.y - d.source.y,\n",
       "                dist = Math.sqrt(deltaX * deltaX + deltaY * deltaY),\n",
       "                normX = deltaX / dist,\n",
       "                normY = deltaY / dist,\n",
       "                sourcePadding = 20,\n",
       "                targetPadding = 29,\n",
       "                sourceX = d.source.x + (sourcePadding * normX),\n",
       "                sourceY = d.source.y + (sourcePadding * normY),\n",
       "                targetX = d.target.x - (targetPadding * normX),\n",
       "                targetY = d.target.y - (targetPadding * normY),\n",
       "                source_doubleX = d.source.x + (20 * normX),\n",
       "                source_doubleY = d.source.y + (20 * normY),\n",
       "                target_doubleX = d.target.x + (50 * normX),\n",
       "                target_doubleY = d.target.y + (50 * normY);\n",
       "\n",
       "                // Commons transitions\n",
       "                if((d.target != d.source) &&\n",
       "                   !(that.existingTrans(d,that.transitions))) {\n",
       "\n",
       "                    return 'M' + sourceX + ',' + sourceY +\n",
       "                        'L' + targetX + ',' + targetY;\n",
       "                }\n",
       "                // Curve for double transitions\n",
       "                else if((d.target != d.source) &&\n",
       "                        that.existingTrans(d,that.transitions)) {\n",
       "\n",
       "                    return \"M\" + (sourceX - (normY*10))  +',' + (sourceY + (normX*10)) +\n",
       "                        \"q\" + ((-normY*20 + (targetX-sourceX)/2)) + ',' +\n",
       "                        ((normX*20 + (targetY-sourceY)/2)) + \" \" +\n",
       "                        (targetX - sourceX) + ',' + (targetY - sourceY);\n",
       "                }\n",
       "                // Curve for the loop\n",
       "                else {\n",
       "                    return \"M\" + (d.source.x-5)  +',' +\n",
       "                        (d.source.y-17) + \" c -50,-50 50,-50 18,-9  \";\n",
       "                }\n",
       "            });\n",
       "\n",
       "            // Set States positions\n",
       "            this.circle.attr('transform', function(d) {\n",
       "                return 'translate(' + d.x + ',' + d.y + ')';\n",
       "            });\n",
       "\n",
       "            // Draw label horizontally on transitions with proper position\n",
       "            this.svg.selectAll(\"text.transitionlabelholder\")\n",
       "                 .attr('transform', function(d) {\n",
       "                     // On commons transitions\n",
       "                     if((d.target != d.source)\n",
       "                        && !(that.existingTrans(d,that.transitions))) {\n",
       "\n",
       "                         return 'translate(' +\n",
       "                             (d.target.x - (d.target.x - d.source.x)/2) + ',' +\n",
       "                             ((d.target.y - (d.target.y - d.source.y)/2)-8) + ')';\n",
       "                     }\n",
       "                     // On doubles transitions\n",
       "                     else if((d.target != d.source) &&\n",
       "                             that.existingTrans(d,that.transitions)){\n",
       "                         var deltaX = d.target.x - d.source.x ,\n",
       "                         deltaY = d.target.y - d.source.y,\n",
       "                         dist = Math.sqrt(deltaX * deltaX + deltaY * deltaY),\n",
       "                         normX = deltaX / dist,\n",
       "                         normY = deltaY / dist,\n",
       "                         sourcePadding = 20,\n",
       "                         targetPadding = 29,\n",
       "                         sourceX = d.source.x + (sourcePadding * normX),\n",
       "                         sourceY = d.source.y + (sourcePadding * normY),\n",
       "                         targetX = d.target.x - (targetPadding * normX),\n",
       "                         targetY = d.target.y - (targetPadding * normY);\n",
       "\n",
       "                         return 'translate(' +\n",
       "                             (sourceX + (-normY*20 + (targetX-sourceX)/2)) + ',' +\n",
       "                             (sourceY + (normX*20 + (targetY-sourceY)/2)) + ')';\n",
       "                     }\n",
       "                     // On loops\n",
       "                     else {\n",
       "                         return 'translate(' + (d.source.x -5) + ',' +\n",
       "                             (d.target.y - 60) + ')';\n",
       "                     }\n",
       "                 });\n",
       "         },\n",
       "\n",
       "        // MOUSE FUNCTIONS\n",
       "        resetMouseVars: function(){\n",
       "            this.mousedown_state = null;\n",
       "            this.mouseup_state = null;\n",
       "            this.mouseover_state = null;\n",
       "            this.mousedown_transition = null;\n",
       "        },\n",
       "        mousedown: function(doc, data, id){\n",
       "            d3.event.preventDefault();\n",
       "\n",
       "            // Active for webkit\n",
       "            this.svg.classed('active', true);\n",
       "\n",
       "            if(d3.event.ctrlKey || this.mousedown_state || this.mousedown_transition) return;\n",
       "            if(!(this.svg.classed('m'))) {\n",
       "                // insert new state at point\n",
       "                var point = d3.mouse(doc);\n",
       "                var lastState = this.lastStateValidId(this.states);\n",
       "                var state = {id: lastState + 1};\n",
       "                state.x = point[0];\n",
       "                state.y = point[1];\n",
       "                var new_states = this.model.get('states').slice();\n",
       "                new_states.push(state);\n",
       "                this.model.set('states', new_states);\n",
       "                this.touch();\n",
       "                this.states = new_states;\n",
       "                this.model.set(\"lastStateId\", this.lastStateId);\n",
       "                this.touch();\n",
       "            }\n",
       "            this.update();\n",
       "        },\n",
       "        mousemove: function(doc, data, id){\n",
       "            if(!this.mousedown_state) return;\n",
       "            // update drag line\n",
       "            if(this.svg.classed('m') == false)\n",
       "                this.drag_line.attr('d', 'M' + this.mousedown_state.x + ',' +\n",
       "                                    this.mousedown_state.y + 'L' +\n",
       "                                    d3.mouse(doc)[0] + ',' + d3.mouse(doc)[1]);\n",
       "            this.update();\n",
       "        },\n",
       "        mouseup: function(doc, data, id){\n",
       "\n",
       "            // add new transition when we are dragging from a state\n",
       "            if(this.mousedown_state && this.mouseover_state) {\n",
       "                this.drag_line.classed('hidden', true)\n",
       "                    .style('marker-end','');\n",
       "                this.mouseup_state = this.mouseover_state;\n",
       "                if(this.mouseup_state === this.mousedown_state) {\n",
       "                    this.resetMouseVars(); return;\n",
       "                }\n",
       "\n",
       "                // add transition and update if already exist\n",
       "                var source, target;\n",
       "                    source = this.mousedown_state;\n",
       "                    target = this.mouseup_state;\n",
       "                var transition;\n",
       "                transition = this.transitions.filter(function(l) {\n",
       "                    return (l.source === source && l.target === target);\n",
       "                })[0];\n",
       "\n",
       "                // a transparent state can't have other transitions\n",
       "                if(!this.isHidden(this.mousedown_state) &&\n",
       "                  !this.isHidden(this.mouseup_state))\n",
       "                {\n",
       "                    if(!transition) {\n",
       "                        transition = {source: source, target: target, label: 'b'};\n",
       "                        var new_transitions = this.model.get('transitions').slice();\n",
       "                        new_transitions.push(transition);\n",
       "                        this.model.set('transitions', new_transitions)\n",
       "                        this.touch();\n",
       "                    }\n",
       "                }\n",
       "\n",
       "                // Reset variable and select the new transition\n",
       "                this.selected_transition = transition;\n",
       "                this.selected_state = null;\n",
       "                this.mousedown_state = null;\n",
       "                this.mouseup_state = null;\n",
       "                this.mousedown_transition = null;\n",
       "                this.update();\n",
       "            }\n",
       "\n",
       "            // because :active only works in WebKit\n",
       "            this.svg.classed('active', false);\n",
       "            this.drag_line.classed('hidden', true);\n",
       "\n",
       "            // clear mouse event vars\n",
       "            this.resetMouseVars();\n",
       "        },\n",
       "\n",
       "        // KEYBOARD FUNCTIONS\n",
       "        spliceTransitionsForState: function(state, transitions){\n",
       "            var toSplice = transitions.filter(function(l) {\n",
       "                return (l.source === state || l.target === state);\n",
       "            });\n",
       "            toSplice.map(function(l) {\n",
       "                transitions.splice(transitions.indexOf(l), 1);\n",
       "            });\n",
       "        },\n",
       "\n",
       "        keydown: function(doc, d , i){\n",
       "\n",
       "            if(this.lastKeyDown !== -1) return;\n",
       "            this.lastKeyDown = d3.event.keyCode;\n",
       "\n",
       "            // m\n",
       "            if(d3.event.keyCode === 77) {\n",
       "                this.circle.call(this.force.drag);\n",
       "                this.svg.classed('m', true);\n",
       "            }\n",
       "\n",
       "            if(!this.selected_state && !this.selected_transition && !this.selected_frame) return;\n",
       "            switch(d3.event.keyCode) {\n",
       "                case 8:\n",
       "                case 66: // key delete\n",
       "                    if(!this.overtext){\n",
       "                        if(this.selected_state) {\n",
       "                            var new_transitions = this.model.get('transitions').slice();\n",
       "                            this.states.splice(this.states.indexOf(this.selected_state), 1);\n",
       "                            this.spliceTransitionsForState(this.selected_state, new_transitions);\n",
       "                            this.model.set('transitions', new_transitions)\n",
       "                            this.touch();\n",
       "                        }\n",
       "                        else if(this.selected_transition && !this.overtext) {\n",
       "                            if(this.selected_transition.target == this.selected_transition.source){\n",
       "                                var new_transitions = this.model.get('transitions').slice();\n",
       "                                new_transitions.splice(new_transitions.indexOf(this.selected_transition), 1);\n",
       "                                this.model.set('transitions', new_transitions)\n",
       "                                this.touch();\n",
       "                            }\n",
       "                            else {\n",
       "                                var new_transitions = this.model.get('transitions').slice();\n",
       "                                new_transitions.splice(new_transitions.indexOf(this.selected_transition), 1);\n",
       "                                this.model.set('transitions', new_transitions)\n",
       "                                this.touch();\n",
       "\n",
       "                            }\n",
       "                        }\n",
       "                        if(!this.overtext) {\n",
       "                            this.selected_transition = null;\n",
       "                            this.selected_state = null;\n",
       "                        }\n",
       "                        this.update();\n",
       "                        break;\n",
       "                    }\n",
       "                case 83: // key S : fix a node\n",
       "                    if(!this.overtext){\n",
       "                        if(this.selected_state) {\n",
       "                            this.selected_state.fixed = true;\n",
       "                        }\n",
       "                    }\n",
       "                    this.update();\n",
       "                    break;\n",
       "                case 70: // key F : create final transition\n",
       "                    if(!(this.isHidden(this.selected_state)) && !this.overtext){\n",
       "                        if(this.selected_state) {\n",
       "                            if(!(this.isFinal(this.selected_state, this.transitions))) {\n",
       "                                var pointx = this.selected_state.x,\n",
       "                                    pointy = this.selected_state.y,\n",
       "                                    state = {id: (this.selected_state.id) + 0.2};\n",
       "                                state.x = pointx +75;\n",
       "                                state.y = pointy;\n",
       "                                var new_states = this.model.get('states').slice();\n",
       "                                new_states.push(state);\n",
       "                                this.model.set('states', new_states);\n",
       "                                this.touch();\n",
       "                                var transition = {source: this.selected_state, target: state, label: ' '};\n",
       "                                var new_transitions = this.model.get('transitions').slice();\n",
       "                                new_transitions.push(transition);\n",
       "                                this.model.set('transitions', new_transitions);\n",
       "                                this.touch();\n",
       "                            }\n",
       "                            else { // delete final transitions\n",
       "                                var new_transitions = this.model.get('transitions').slice();\n",
       "                                this.spliceTransitionsForState(this.findFinalState(this.selected_state, this.states), new_transitions);\n",
       "                                this.states.splice(this.states.indexOf(this.findFinalState(this.selected_state, this.states)), 1);\n",
       "                                this.model.set('transitions', new_transitions);\n",
       "                                this.touch();\n",
       "                            }\n",
       "                        }\n",
       "                    this.update();\n",
       "                    break;\n",
       "                    }\n",
       "                case 73: // key I : create initial transition\n",
       "                    if(!(this.isHidden(this.selected_state)) && !this.overtext){\n",
       "                        if(this.selected_state) {\n",
       "                            if(!(this.isInitial(this.selected_state, this.transitions))) {\n",
       "                                var pointx = this.selected_state.x,\n",
       "                                    pointy = this.selected_state.y,\n",
       "                                    state = {id: (this.selected_state.id) + 0.1};\n",
       "                                state.x = pointx -75;\n",
       "                                state.y = pointy;\n",
       "                                var new_states = this.model.get('states').slice();\n",
       "                                new_states.push(state);\n",
       "                                this.model.set('states', new_states);\n",
       "                                this.touch();\n",
       "                                var transition = {source: state, target: this.selected_state, label: ' '};\n",
       "                                var new_transitions = this.model.get('transitions').slice();\n",
       "                                new_transitions.push(transition);\n",
       "                                this.model.set('transitions', new_transitions);\n",
       "                                this.touch();\n",
       "                            }\n",
       "                            else { // delete initial transitions\n",
       "                                var new_transitions = this.model.get('transitions').slice();\n",
       "                                this.spliceTransitionsForState(this.findInitialState(this.selected_state, this.states), new_transitions);\n",
       "                                this.states.splice(this.states.indexOf(this.findInitialState(this.selected_state, this.states)), 1);\n",
       "                                this.model.set('transitions', new_transitions);\n",
       "                                this.touch();\n",
       "                            }\n",
       "                        }\n",
       "                    this.update();\n",
       "                    break;\n",
       "                    }\n",
       "                case 76: // key L : create loop on node\n",
       "                    if(!(this.isHidden(this.selected_state)) && !this.overtext){\n",
       "                        if(this.selected_state && !(this.isLoop(this.selected_state, this.transitions))) {\n",
       "                            var loop = {source: this.selected_state, target: this.selected_state, label:'a', short: false};\n",
       "                            var new_transitions = this.model.get('transitions').slice();\n",
       "                            new_transitions.push(loop);\n",
       "                            this.model.set('transitions', new_transitions);\n",
       "                            this.touch();\n",
       "                        }\n",
       "                    this.update();\n",
       "                    break;\n",
       "                    }\n",
       "            }\n",
       "        },\n",
       "\n",
       "        keyup: function(doc, d, i){\n",
       "              this.lastKeyDown = -1;\n",
       "              // m\n",
       "              if(d3.event.keyCode === 77) {\n",
       "                this.circle\n",
       "                  .on('mousedown.drag', null)\n",
       "                  .on('touchstart.drag', null);\n",
       "                this.svg.classed('m', false);\n",
       "              }\n",
       "        },\n",
       "\n",
       "        // METHODS\n",
       "        idToState: function(id){\n",
       "            for(var j=0; j <= this.states.length -1; j++)\n",
       "                {\n",
       "                    if(this.states[j].id == id)\n",
       "                        {\n",
       "                            return this.states[j];\n",
       "                        }\n",
       "                }\n",
       "        },\n",
       "        existingTrans: function(transition, transitions){\n",
       "            for(var i=0; i<= transitions.length -1; i++){\n",
       "                if((transitions[i].source == transition.target) &&\n",
       "                  (transitions[i].target == transition.source)){\n",
       "                    return true;\n",
       "                    }\n",
       "                }\n",
       "            return false;\n",
       "        },\n",
       "        lastStateValidId: function(states){\n",
       "            for(var t = states.length -1; t>=0; t--)\n",
       "                    {\n",
       "                        if((parseFloat((states[t].id)) == parseInt((states[t].id)))\n",
       "                           && !isNaN((states[t].id)))\n",
       "                            {\n",
       "                                return states[t].id;\n",
       "                            }\n",
       "                    }\n",
       "\n",
       "        },\n",
       "        findFinalState: function(state, states){\n",
       "            for(var i = 0; i <= states.length -1; i++)\n",
       "                {\n",
       "                    if(states[i].id.toString() == (state.id.toString() + '.2'))\n",
       "                        {\n",
       "                            return states[i];\n",
       "                        }\n",
       "                }\n",
       "        },\n",
       "        findInitialState: function(state, states){\n",
       "            for(var i = 0; i <= states.length -1; i++)\n",
       "                {\n",
       "                    if(states[i].id.toString() == (state.id.toString() + '.1'))\n",
       "                        {\n",
       "                            return states[i];\n",
       "                        }\n",
       "                }\n",
       "        },\n",
       "        isInitial: function(state, transitions){\n",
       "                for(var i=0; i<= transitions.length -1; i++)\n",
       "                    {\n",
       "                        if(transitions[i].source.id == state.id ||\n",
       "                           transitions[i].target.id == state.id)\n",
       "                            {\n",
       "                                if(!((parseFloat((transitions[i].source.id)) ==\n",
       "                                      parseInt((transitions[i].source.id))) &&\n",
       "                                     !isNaN((transitions[i].source.id))) )\n",
       "                                    {\n",
       "                                        return true;\n",
       "                                    }\n",
       "                            }\n",
       "                    }\n",
       "                return false;\n",
       "        },\n",
       "        isFinal: function(state, transitions){\n",
       "                for(var i=0; i<= transitions.length -1; i++)\n",
       "                    {\n",
       "                        if(transitions[i].source.id == state.id ||\n",
       "                           transitions[i].target.id == state.id)\n",
       "                            {\n",
       "                                if(!((parseFloat((transitions[i].target.id)) ==\n",
       "                                      parseInt((transitions[i].target.id))) &&\n",
       "                                     !isNaN((transitions[i].target.id))) )\n",
       "                                    {\n",
       "                                        return true;\n",
       "                                    }\n",
       "                            }\n",
       "                    }\n",
       "                return false;\n",
       "        },\n",
       "        isHidden: function(state){\n",
       "                if(!(parseFloat(state.id) == parseInt(state.id) && !isNaN(state.id)))\n",
       "                    {\n",
       "                        return true;\n",
       "                    }\n",
       "                else{ return false;}\n",
       "        },\n",
       "        isShort: function(transition){\n",
       "                if(!((parseFloat((transition.target.id)) ==\n",
       "                      parseInt((transition.target.id))) &&\n",
       "                     !isNaN((transition.target.id)))\n",
       "                  || !((parseFloat((transition.source.id)) ==\n",
       "                        parseInt((transition.source.id))) &&\n",
       "                       !isNaN((transition.source.id))))\n",
       "                    {\n",
       "                        return true;\n",
       "                    }\n",
       "                else {return false;}\n",
       "\n",
       "        },\n",
       "        isLoop: function(state, transitions){\n",
       "                for(var i =0; i <= transitions.length -1; i++)\n",
       "                    {\n",
       "                        if(transitions[i].target.id == transitions[i].source.id)\n",
       "                            {\n",
       "                                if(transitions[i].target.id == state.id)\n",
       "                                    {\n",
       "                                        return true;\n",
       "                                    }\n",
       "                            }\n",
       "                    }\n",
       "                return false;\n",
       "        },\n",
       "        idToNode: function(id){\n",
       "            for(var j=0; j <= this.states.length -1; j++)\n",
       "                {\n",
       "                    if(this.states[j].id == id)\n",
       "                        {\n",
       "                            return this.states[j];\n",
       "                        }\n",
       "                }\n",
       "        },\n",
       "        completeTransitions: function(transitions){\n",
       "                for(var i =0; i <= transitions.length -1; i++)\n",
       "                    {\n",
       "                        if(transitions[i].length != 3)\n",
       "                            {\n",
       "                                if(typeof(transitions[i].source) === 'undefined')\n",
       "                                    {\n",
       "                                        //(transitions[i].target)+'.1'\n",
       "                                        var state = {id:((transitions[i].target)+'.1') };\n",
       "\n",
       "                                        this.states.push(state);\n",
       "                                        var trans = {source: state.id.toString()\n",
       "                                                     , target:transitions[i].target,\n",
       "                                                     label:transitions[i].label};\n",
       "                                        transitions.splice(i,1);\n",
       "                                        transitions.push(trans);\n",
       "                                        i --;\n",
       "                                        this.model.set('transitions', transitions);\n",
       "                                        this.touch();\n",
       "                                        this.model.set('states', this.states);\n",
       "                                        this.touch();\n",
       "                                    }\n",
       "                                else if(typeof(transitions[i].target) === 'undefined')\n",
       "                                    {\n",
       "                                        //(transitions[i].source)+'.2'\n",
       "                                        var state = {id:((transitions[i].source)+'.2')};\n",
       "                                        this.states.push(state);\n",
       "                                        var trans = {source: transitions[i].source,\n",
       "                                                     target: state.id.toString(),\n",
       "                                                     label:transitions[i].label}\n",
       "                                        transitions.splice(i,1);\n",
       "                                        transitions.push(trans);\n",
       "                                        i --;\n",
       "                                        this.model.set('transitions', transitions);\n",
       "                                        this.touch();\n",
       "                                        this.model.set('states', this.states);\n",
       "                                        this.touch();\n",
       "                                    }\n",
       "                            }\n",
       "                    }\n",
       "            return transitions;\n",
       "        },\n",
       "\n",
       "        // UPDATE THE VIEW\n",
       "        update: function(){\n",
       "\n",
       "            var colors = d3.scale.category10();\n",
       "            // Update force\n",
       "            this.force.nodes(this.states)\n",
       "                .links(this.transitions)\n",
       "\n",
       "            // Closure\n",
       "            var that = this;\n",
       "\n",
       "            // Update the datas on model\n",
       "            this.states = this.model.get('states');\n",
       "            this.transitions = this.model.get('transitions');\n",
       "            this.lastStateId = this.lastStateValidId(this.states);\n",
       "            this.model.set(\"lastStateId\", this.lastStateId);\n",
       "            this.model.set('states', this.states);\n",
       "            this.touch();\n",
       "\n",
       "            // Update the frame for label\n",
       "            this.frame.selectAll('input').attr('value', function(){\n",
       "                    if(that.selected_transition)\n",
       "                        this.value = that.selected_transition.label;\n",
       "                    else\n",
       "                        this.value = '';\n",
       "            })\n",
       "            this.frame.attr('value', function(){\n",
       "                    if(that.selected_transition)\n",
       "                        this.value = that.selected_transition.label;\n",
       "                    else\n",
       "                        this.value = '';\n",
       "            })\n",
       "                .on(\"mousedown\", function() {\n",
       "                    this.focus();\n",
       "                    that.overtext = true;\n",
       "                    that.selected_frame = true;\n",
       "                    that.update();\n",
       "                    if(that.selected_transition)\n",
       "                        this.value = that.selected_transition.label;\n",
       "                    else\n",
       "                        this.value = '';\n",
       "                })\n",
       "                .on(\"keypress\", function() {\n",
       "\n",
       "                    // IE fix\n",
       "                    if (!d3.event)\n",
       "                        d3.event = window.event;\n",
       "\n",
       "                    var e = d3.event;\n",
       "                    if (e.keyCode == 13)\n",
       "                    {\n",
       "                        if (typeof(e.cancelBubble) !== 'undefined') // IE\n",
       "                            e.cancelBubble = true;\n",
       "                        if (e.stopPropagation)\n",
       "                            e.stopPropagation();\n",
       "                        e.preventDefault();\n",
       "\n",
       "                        var txt = that.frame.node().value;\n",
       "\n",
       "                        that.selected_transition.label = txt;\n",
       "\n",
       "                        // trick to update automatically the label for vcsn\n",
       "                        var old_transitions = that.transitions;\n",
       "                        var last_transition = that.transitions[that.transitions.length -1]\n",
       "                        var new_transitions1 = that.model.get('transitions').slice();\n",
       "                        new_transitions1.splice(that.transitions.indexOf(that.transitions[that.transitions.length -1]),1)\n",
       "                        that.model.set('transitions', new_transitions1)\n",
       "                        var new_transitions2 = that.model.get('transitions').slice();\n",
       "                        new_transitions2.push(last_transition)\n",
       "                        that.model.set('transitions', new_transitions2)\n",
       "                        that.touch()\n",
       "\n",
       "                        that.model.set('transitions', old_transitions);\n",
       "                        that.touch();\n",
       "                        that.selected_transition = null;\n",
       "                        this.value = ''\n",
       "                        that.update();\n",
       "                    }\n",
       "                })\n",
       "\n",
       "            // Define elements groups and bound them data\n",
       "            this.path = this.path.data(this.transitions, function(d) {\n",
       "                return [d.source.id, d.target.id];});\n",
       "            this.circle = this.circle.data(this.states, function(d) {\n",
       "                return d.id;});\n",
       "\n",
       "            // TRANSITIONS\n",
       "            // Update existing transition\n",
       "            this.path.selectAll('path')\n",
       "                .classed('selected', function(d) {return d == that.selected_transition;})\n",
       "                .style('marker-end', function(d) {return 'url(#end-arrow)'; })\n",
       "                .style('stroke-dasharray', function(d) {\n",
       "                    if (d == that.selected_transition) {return \"10,2\";}\n",
       "                    else {return'';}})\n",
       "            // Enter new transition\n",
       "            var p = this.path.enter().append('svg:g');\n",
       "            p.append('svg:path')\n",
       "                .attr('class', 'transition')\n",
       "                .style('fill', 'none')\n",
       "                .style('stroke', '#000')\n",
       "                .style('stroke-width', '1.75px')\n",
       "                .style('cursor', 'pointer')\n",
       "                .classed('selected', function(d) { return d === that.selected_transition; })\n",
       "                .style('marker-end', function(d) {return 'url(#end-arrow)'; })\n",
       "                .style('stroke-dasharray', function(d) {\n",
       "                    if (d == that.selected_transition) {return \"10,2\";}\n",
       "                    else {return'';}})\n",
       "                .on('mousedown', function(d) {\n",
       "                      if(d3.event.keyCode === 77) return;\n",
       "                      // select transition\n",
       "                      that.mousedown_transition = d;\n",
       "                      if(that.mousedown_transition === that.selected_transition)\n",
       "                          that.selected_transition = null;\n",
       "                      else that.selected_transition = that.mousedown_transition;\n",
       "                      that.selected_state = null;\n",
       "                      that.selected_frame = false;\n",
       "                      that.update();\n",
       "                })\n",
       "\n",
       "            // Exit old transitions\n",
       "            this.path.exit().remove();\n",
       "\n",
       "            // TRANSITIONS LABELS\n",
       "            // Update existing labels\n",
       "            this.path.selectAll('text')\n",
       "                .text(function(d){return d.label});\n",
       "            // Show transition labels\n",
       "            p.append('svg:text')\n",
       "            .attr(\"class\", \"transitionlabelholder\")\n",
       "            .style(\"font-size\", \"18px\")\n",
       "            //.style(\"fill\", \"#00CCFF\")\n",
       "            .text(function(d) {return d.label});\n",
       "\n",
       "            // STATES\n",
       "            // Update existing states\n",
       "            this.circle.selectAll('circle')\n",
       "                .style('fill', function(d) {\n",
       "                    if (that.isHidden(d)) {return \"transparent\";}\n",
       "                    else {return (d === that.selected_state) ?\n",
       "                          d3.rgb(colors(d.id)).brighter().toString()\n",
       "                          : colors(d.id); }})\n",
       "                .style('stroke', function(d) {\n",
       "                    if (that.isHidden(d)) {return \"transparent\";}\n",
       "                    else {return d3.rgb(colors(d.id)).darker().toString(); }})\n",
       "            // Enter new states\n",
       "            var g = this.circle.enter().append('svg:g')\n",
       "            g.append('svg:circle')\n",
       "                .attr('class', 'state')\n",
       "                .attr('r', 20)\n",
       "                .style('stroke-width', '1.5px')\n",
       "                .style('cursor', 'pointer')\n",
       "                .style('fill', function(d) {\n",
       "                    if (that.isHidden(d)) {return \"transparent\";}\n",
       "                    else {return (d === that.selected_state) ?\n",
       "                          d3.rgb(colors(d.id)).brighter().toString()\n",
       "                          : colors(d.id); }})\n",
       "                .style('stroke', function(d) {\n",
       "                    if (that.isHidden(d)) {return \"transparent\";}\n",
       "                    else {return d3.rgb(colors(d.id)).darker().toString(); }})\n",
       "                .on('mouseover', function(d) {\n",
       "                    that.mouseover_state = d;\n",
       "                    if(that.isHidden(d)) {\n",
       "                        d3.select(this).style('stroke','black')\n",
       "                    }\n",
       "                })\n",
       "                .on('mouseout', function(d) {\n",
       "                    if(that.isHidden(d)) {\n",
       "                        d3.select(this).style('stroke','transparent')\n",
       "                    }\n",
       "                })\n",
       "                .on('mousedown', function(d) {\n",
       "                    d3.event.preventDefault();\n",
       "                    if(d3.event.keyCode === 77) return;\n",
       "                    // select state\n",
       "                    that.mousedown_state = d;\n",
       "                    if(that.mousedown_state === that.selected_state)\n",
       "                        that.selected_state = null;\n",
       "                    else {that.selected_state = that.mousedown_state;}\n",
       "                    that.selected_transition = null;\n",
       "                    // reposition drag line\n",
       "                    if(!that.isHidden(that.mousedown_state)  &&\n",
       "                       that.svg.classed('m') == false)\n",
       "                    {\n",
       "                        that.drag_line\n",
       "                            .style('marker-end', 'url(#end-arrow)')\n",
       "                            .classed('hidden', false)\n",
       "                            .attr('d', 'M' + that.mousedown_state.x + ',' +\n",
       "                                  that.mousedown_state.y + 'L' +\n",
       "                                  that.mousedown_state.x + ',' +\n",
       "                                  that.mousedown_state.y);\n",
       "                    }\n",
       "                    that.selected_frame = false;\n",
       "                    that.update();\n",
       "                })\n",
       "\n",
       "            // Exit old states\n",
       "            this.circle.exit().remove();\n",
       "\n",
       "            // STATES ID\n",
       "            // Update states ID\n",
       "            this.circle.selectAll('text')\n",
       "                .text(function(d) {\n",
       "                    if(that.isHidden(d))\n",
       "                    {return '';}\n",
       "                    else\n",
       "                    { return d.id;}\n",
       "                })\n",
       "\n",
       "            // Show states ID\n",
       "            g.append('svg:text')\n",
       "                .attr('x', 0)\n",
       "                .attr('y', 4)\n",
       "                .attr('class', 'id')\n",
       "            .style('font', '15px sans-serif')\n",
       "                .style(\"text-anchor\", \"middle\")\n",
       "                .style(\"fill\", function(d) {\n",
       "                    if(that.isHidden(d)) return \"transparent\";\n",
       "                    else return \"white\";})\n",
       "                .text(function(d) { return d.id; })\n",
       "                .style('cursor', 'pointer')\n",
       "                .on('mousedown', function(d) {\n",
       "                    d3.event.preventDefault();\n",
       "                    if(d3.event.keyCode === 77) return;\n",
       "                    // select state\n",
       "                    that.mousedown_state = d;\n",
       "                    if(that.mousedown_state === that.selected_state)\n",
       "                        that.selected_state = null;\n",
       "                    else {that.selected_state = that.mousedown_state;}\n",
       "                    that.selected_transition = null;\n",
       "                    // reposition drag line\n",
       "                    if(!that.isHidden(that.mousedown_state)  &&\n",
       "                       that.svg.classed('m') == false)\n",
       "                    {\n",
       "                        that.drag_line\n",
       "                            .style('marker-end', 'url(#end-arrow)')\n",
       "                            .classed('hidden', false)\n",
       "                            .attr('d', 'M' + that.mousedown_state.x + ',' +\n",
       "                                  that.mousedown_state.y + 'L' +\n",
       "                                  that.mousedown_state.x + ',' +\n",
       "                                  that.mousedown_state.y);\n",
       "                    }\n",
       "                    that.selected_frame = false;\n",
       "                    that.update();\n",
       "                })\n",
       "            this.force.start();\n",
       "        }\n",
       "    });\n",
       "\n",
       "    return {\n",
       "        AutomatonView: AutomatonView\n",
       "    }\n",
       "});"
      ],
      "text/plain": [
       "<IPython.core.display.Javascript object>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "%%javascript\n",
    "require.undef('AutomatonD3Widget');\n",
    "\n",
    "define('AutomatonD3Widget',\n",
    "       ['jupyter-js-widgets', 'd3'],\n",
    "       function(widgets, d3) {\n",
    "    var AutomatonView = widgets.DOMWidgetView.extend({\n",
    "\n",
    "        // RENDER THE VIEW\n",
    "        render: function(){\n",
    "            // Here we define variables with 'this' to call them from everywhere\n",
    "\n",
    "            // Editable text for labels\n",
    "            this.svg_for_frame = d3.select(this.el).append('svg')\n",
    "                .attr('width', 960)\n",
    "                .attr('height', 100)\n",
    "            this.frame_title = this.svg_for_frame.append('svg:text')\n",
    "                .text('Label: ')\n",
    "                .attr('x', 25)\n",
    "                .attr('y', 25)\n",
    "            this.frame = this.svg_for_frame.append('foreignObject')\n",
    "                .attr('x', 85)\n",
    "                .attr('y', 10)\n",
    "                .attr(\"width\", 800)\n",
    "                .attr(\"height\", 50)\n",
    "                .append(\"xhtml:form\")\n",
    "                .attr('class', 'frame')\n",
    "                .append('input')\n",
    "\n",
    "            // Attributs of the SVG\n",
    "            this.width =960, this.height = 500;\n",
    "\n",
    "            // Render the svg\n",
    "            this.svg = d3.select(this.el).append('svg')\n",
    "                .attr('width', this.width)\n",
    "                .attr('height', this.height)\n",
    "                .style('userSelect' , \"none\")\n",
    "                .style('webkiUserSelect' , \"none\")\n",
    "                .style('MozUserSelect' , \"none\")\n",
    "                .style('oUserSelect', 'none')\n",
    "                .style('msUserSelect', 'none')\n",
    "                .style('draggable', 'false')\n",
    "                .classed('m', false)\n",
    "                .style('cursor', 'crosshair')\n",
    "\n",
    "            // Initialize datas\n",
    "            this.states = this.model.get('states');\n",
    "            var trans = this.model.get('transitions');\n",
    "            this.transitions = this.completeTransitions(trans);\n",
    "            this.lastStateId = this.states[this.states.length - 1].id;\n",
    "\n",
    "            // Define here to avoid recreation in every update\n",
    "            this.path = this.svg.append('svg:g').selectAll('g');\n",
    "            this.circle = this.svg.append('svg:g').selectAll('g');\n",
    "\n",
    "            // Lines displayed when draggind new states\n",
    "            this.drag_line = this.svg.append('svg:path')\n",
    "                .style('fill', 'none')\n",
    "                .style('stroke', '#000')\n",
    "                .style('stroke-width', '1.50px')\n",
    "                .attr('class', 'transition dragline hidden')\n",
    "                .attr('d', 'M0,0L0,0');\n",
    "\n",
    "            // Arrow for transitions\n",
    "            this.svg.append('svg:defs').append('svg:marker')\n",
    "                .attr('id', 'end-arrow')\n",
    "                .attr('viewBox', '-3 -10 20 20')\n",
    "                .attr('refX', 2)\n",
    "                .attr('markerWidth', 6)\n",
    "                .attr('markerHeight', 6)\n",
    "                .attr('orient', 'auto')\n",
    "              .append('svg:path')\n",
    "                .attr('d', 'M-3,12 l22,-12 l-22,-12 l7,12 Z')\n",
    "                .attr('fill', '#000');\n",
    "\n",
    "            // Closure\n",
    "            var that = this;\n",
    "\n",
    "            // Catch source and target with their ID\n",
    "            for(var c = 0; c <= this.transitions.length -1; c++)\n",
    "                {\n",
    "                    this.transitions[c].source = this.idToState(this.transitions[c].source);\n",
    "                    this.transitions[c].target = this.idToState(this.transitions[c].target);\n",
    "                }\n",
    "\n",
    "            // Force layout for physical simulation\n",
    "            this.force = d3.layout.force()\n",
    "                .nodes(this.states)\n",
    "                .links(this.transitions)\n",
    "                .size([this.width, this.height])\n",
    "                .linkDistance(function(d) {\n",
    "                    if(that.isShort(d)) {return 75;}\n",
    "                    else { return 175;}\n",
    "                })\n",
    "                .charge(function(d) {\n",
    "                    if(that.isHidden(d)) {return -100;}\n",
    "                    else {return -750;}\n",
    "                })\n",
    "                .on('tick', function(d,i){ that.stepForce(this, d, i);})\n",
    "\n",
    "            // Listen for mouse and keyboard events on the whole document\n",
    "            this.svg.on('mousedown', function(d,i){ that.mousedown(this, d, i);})\n",
    "                .on('mousemove', function(d,i){ that.mousemove(this, d, i);})\n",
    "                .on('mouseup', function(d,i){ that.mouseup(this, d, i);})\n",
    "            // Active keyboard action and disable notebook shortcuts when we are\n",
    "            // over the widget\n",
    "                .on(\"mouseover\", function(d,i) {\n",
    "                    // Disable some notebook key shortcuts\n",
    "                    var cmd = IPython.keyboard_manager.command_shortcuts;\n",
    "                    cmd.remove_shortcut('i');\n",
    "                    cmd.remove_shortcut('l');\n",
    "                    cmd.remove_shortcut('f');\n",
    "                    cmd.remove_shortcut('m');\n",
    "                    cmd.remove_shortcut('s');\n",
    "\n",
    "                    d3.select(window).on('keydown', function(d,i){\n",
    "                        that.keydown(this, d, i);})\n",
    "                        .on('keyup',  function(d,i){\n",
    "                            that.keyup(this, d, i);});\n",
    "                })\n",
    "            // if we leave the widget area, then disable keyboard action and\n",
    "            // reactive notebook shortcuts\n",
    "                .on(\"mouseout\", function(d,i) {\n",
    "                    // reset selections if we leave widget\n",
    "                    // that.selected_transition = null;\n",
    "                    // that.selected_state = null;\n",
    "                    // that.mousedown_state = null;\n",
    "                    // that.mouseup_state = null;\n",
    "                    // that.mousedown_transition = null;\n",
    "                    var cmd = IPython.keyboard_manager.command_shortcuts;\n",
    "                        cmd.add_shortcut('i', {\n",
    "                            help    : 'interrupt kernel (press twice)',\n",
    "                            help_index : 'ha',\n",
    "                            count: 2,\n",
    "                            handler : function (event) {\n",
    "                                IPython.notebook.kernel.interrupt();\n",
    "                                return false;\n",
    "                            }\n",
    "                        });\n",
    "                        cmd.add_shortcut('l', {\n",
    "                            help    : 'toggle line numbers',\n",
    "                            help_index : 'ga',\n",
    "                            handler : function (event) {\n",
    "                                IPython.notebook.cell_toggle_line_numbers();\n",
    "                                return false;\n",
    "                            }\n",
    "                        });\n",
    "                        cmd.add_shortcut('m', {\n",
    "                            help    : 'to markdown',\n",
    "                            help_index : 'cb',\n",
    "                            handler : function (event) {\n",
    "                                IPython.notebook.to_markdown();\n",
    "                                return false;\n",
    "                            }\n",
    "                        });\n",
    "                        cmd.add_shortcut('s', {\n",
    "                            help    : 'save notebook',\n",
    "                            help_index : 'fa',\n",
    "                            handler : function (event) {\n",
    "                                IPython.notebook.save_checkpoint();\n",
    "                                return false;\n",
    "                            }\n",
    "                        });\n",
    "                    d3.select(window).on('keydown', function() {\n",
    "                        return null})\n",
    "                        .on('keyup', function() {return null;});\n",
    "                });\n",
    "\n",
    "            this.hackUpdate();\n",
    "        },\n",
    "\n",
    "        hackUpdate: function(){\n",
    "            // Little hack to update model without touch the view\n",
    "            // Launched at the begginning avoid conversion error\n",
    "            // Between VCSN and D3\n",
    "\n",
    "            var old_transitions = this.transitions\n",
    "            var transition = {'source': '0', 'target':'0', 'label': ''}\n",
    "            var new_transitions = this.model.get('transitions').slice();\n",
    "            new_transitions.push(transition);\n",
    "            this.model.set('transitions', new_transitions);\n",
    "            this.touch();\n",
    "\n",
    "            this.model.set('transitions', old_transitions);\n",
    "            this.touch();\n",
    "\n",
    "            this.update();\n",
    "        },\n",
    "\n",
    "        // SET POSITIONS OF EVERY ELEMENTS\n",
    "        stepForce: function(doc, da, i) {\n",
    "\n",
    "            var that = this;\n",
    "            // Draw transitions with proper padding from state centers\n",
    "            this.path.selectAll('path').attr('d', function(d) {\n",
    "\n",
    "                var deltaX = d.target.x - d.source.x ,\n",
    "                deltaY = d.target.y - d.source.y,\n",
    "                dist = Math.sqrt(deltaX * deltaX + deltaY * deltaY),\n",
    "                normX = deltaX / dist,\n",
    "                normY = deltaY / dist,\n",
    "                sourcePadding = 20,\n",
    "                targetPadding = 29,\n",
    "                sourceX = d.source.x + (sourcePadding * normX),\n",
    "                sourceY = d.source.y + (sourcePadding * normY),\n",
    "                targetX = d.target.x - (targetPadding * normX),\n",
    "                targetY = d.target.y - (targetPadding * normY),\n",
    "                source_doubleX = d.source.x + (20 * normX),\n",
    "                source_doubleY = d.source.y + (20 * normY),\n",
    "                target_doubleX = d.target.x + (50 * normX),\n",
    "                target_doubleY = d.target.y + (50 * normY);\n",
    "\n",
    "                // Commons transitions\n",
    "                if((d.target != d.source) &&\n",
    "                   !(that.existingTrans(d,that.transitions))) {\n",
    "\n",
    "                    return 'M' + sourceX + ',' + sourceY +\n",
    "                        'L' + targetX + ',' + targetY;\n",
    "                }\n",
    "                // Curve for double transitions\n",
    "                else if((d.target != d.source) &&\n",
    "                        that.existingTrans(d,that.transitions)) {\n",
    "\n",
    "                    return \"M\" + (sourceX - (normY*10))  +',' + (sourceY + (normX*10)) +\n",
    "                        \"q\" + ((-normY*20 + (targetX-sourceX)/2)) + ',' +\n",
    "                        ((normX*20 + (targetY-sourceY)/2)) + \" \" +\n",
    "                        (targetX - sourceX) + ',' + (targetY - sourceY);\n",
    "                }\n",
    "                // Curve for the loop\n",
    "                else {\n",
    "                    return \"M\" + (d.source.x-5)  +',' +\n",
    "                        (d.source.y-17) + \" c -50,-50 50,-50 18,-9  \";\n",
    "                }\n",
    "            });\n",
    "\n",
    "            // Set States positions\n",
    "            this.circle.attr('transform', function(d) {\n",
    "                return 'translate(' + d.x + ',' + d.y + ')';\n",
    "            });\n",
    "\n",
    "            // Draw label horizontally on transitions with proper position\n",
    "            this.svg.selectAll(\"text.transitionlabelholder\")\n",
    "                 .attr('transform', function(d) {\n",
    "                     // On commons transitions\n",
    "                     if((d.target != d.source)\n",
    "                        && !(that.existingTrans(d,that.transitions))) {\n",
    "\n",
    "                         return 'translate(' +\n",
    "                             (d.target.x - (d.target.x - d.source.x)/2) + ',' +\n",
    "                             ((d.target.y - (d.target.y - d.source.y)/2)-8) + ')';\n",
    "                     }\n",
    "                     // On doubles transitions\n",
    "                     else if((d.target != d.source) &&\n",
    "                             that.existingTrans(d,that.transitions)){\n",
    "                         var deltaX = d.target.x - d.source.x ,\n",
    "                         deltaY = d.target.y - d.source.y,\n",
    "                         dist = Math.sqrt(deltaX * deltaX + deltaY * deltaY),\n",
    "                         normX = deltaX / dist,\n",
    "                         normY = deltaY / dist,\n",
    "                         sourcePadding = 20,\n",
    "                         targetPadding = 29,\n",
    "                         sourceX = d.source.x + (sourcePadding * normX),\n",
    "                         sourceY = d.source.y + (sourcePadding * normY),\n",
    "                         targetX = d.target.x - (targetPadding * normX),\n",
    "                         targetY = d.target.y - (targetPadding * normY);\n",
    "\n",
    "                         return 'translate(' +\n",
    "                             (sourceX + (-normY*20 + (targetX-sourceX)/2)) + ',' +\n",
    "                             (sourceY + (normX*20 + (targetY-sourceY)/2)) + ')';\n",
    "                     }\n",
    "                     // On loops\n",
    "                     else {\n",
    "                         return 'translate(' + (d.source.x -5) + ',' +\n",
    "                             (d.target.y - 60) + ')';\n",
    "                     }\n",
    "                 });\n",
    "         },\n",
    "\n",
    "        // MOUSE FUNCTIONS\n",
    "        resetMouseVars: function(){\n",
    "            this.mousedown_state = null;\n",
    "            this.mouseup_state = null;\n",
    "            this.mouseover_state = null;\n",
    "            this.mousedown_transition = null;\n",
    "        },\n",
    "        mousedown: function(doc, data, id){\n",
    "            d3.event.preventDefault();\n",
    "\n",
    "            // Active for webkit\n",
    "            this.svg.classed('active', true);\n",
    "\n",
    "            if(d3.event.ctrlKey || this.mousedown_state || this.mousedown_transition) return;\n",
    "            if(!(this.svg.classed('m'))) {\n",
    "                // insert new state at point\n",
    "                var point = d3.mouse(doc);\n",
    "                var lastState = this.lastStateValidId(this.states);\n",
    "                var state = {id: lastState + 1};\n",
    "                state.x = point[0];\n",
    "                state.y = point[1];\n",
    "                var new_states = this.model.get('states').slice();\n",
    "                new_states.push(state);\n",
    "                this.model.set('states', new_states);\n",
    "                this.touch();\n",
    "                this.states = new_states;\n",
    "                this.model.set(\"lastStateId\", this.lastStateId);\n",
    "                this.touch();\n",
    "            }\n",
    "            this.update();\n",
    "        },\n",
    "        mousemove: function(doc, data, id){\n",
    "            if(!this.mousedown_state) return;\n",
    "            // update drag line\n",
    "            if(this.svg.classed('m') == false)\n",
    "                this.drag_line.attr('d', 'M' + this.mousedown_state.x + ',' +\n",
    "                                    this.mousedown_state.y + 'L' +\n",
    "                                    d3.mouse(doc)[0] + ',' + d3.mouse(doc)[1]);\n",
    "            this.update();\n",
    "        },\n",
    "        mouseup: function(doc, data, id){\n",
    "\n",
    "            // add new transition when we are dragging from a state\n",
    "            if(this.mousedown_state && this.mouseover_state) {\n",
    "                this.drag_line.classed('hidden', true)\n",
    "                    .style('marker-end','');\n",
    "                this.mouseup_state = this.mouseover_state;\n",
    "                if(this.mouseup_state === this.mousedown_state) {\n",
    "                    this.resetMouseVars(); return;\n",
    "                }\n",
    "\n",
    "                // add transition and update if already exist\n",
    "                var source, target;\n",
    "                    source = this.mousedown_state;\n",
    "                    target = this.mouseup_state;\n",
    "                var transition;\n",
    "                transition = this.transitions.filter(function(l) {\n",
    "                    return (l.source === source && l.target === target);\n",
    "                })[0];\n",
    "\n",
    "                // a transparent state can't have other transitions\n",
    "                if(!this.isHidden(this.mousedown_state) &&\n",
    "                  !this.isHidden(this.mouseup_state))\n",
    "                {\n",
    "                    if(!transition) {\n",
    "                        transition = {source: source, target: target, label: 'b'};\n",
    "                        var new_transitions = this.model.get('transitions').slice();\n",
    "                        new_transitions.push(transition);\n",
    "                        this.model.set('transitions', new_transitions)\n",
    "                        this.touch();\n",
    "                    }\n",
    "                }\n",
    "\n",
    "                // Reset variable and select the new transition\n",
    "                this.selected_transition = transition;\n",
    "                this.selected_state = null;\n",
    "                this.mousedown_state = null;\n",
    "                this.mouseup_state = null;\n",
    "                this.mousedown_transition = null;\n",
    "                this.update();\n",
    "            }\n",
    "\n",
    "            // because :active only works in WebKit\n",
    "            this.svg.classed('active', false);\n",
    "            this.drag_line.classed('hidden', true);\n",
    "\n",
    "            // clear mouse event vars\n",
    "            this.resetMouseVars();\n",
    "        },\n",
    "\n",
    "        // KEYBOARD FUNCTIONS\n",
    "        spliceTransitionsForState: function(state, transitions){\n",
    "            var toSplice = transitions.filter(function(l) {\n",
    "                return (l.source === state || l.target === state);\n",
    "            });\n",
    "            toSplice.map(function(l) {\n",
    "                transitions.splice(transitions.indexOf(l), 1);\n",
    "            });\n",
    "        },\n",
    "\n",
    "        keydown: function(doc, d , i){\n",
    "\n",
    "            if(this.lastKeyDown !== -1) return;\n",
    "            this.lastKeyDown = d3.event.keyCode;\n",
    "\n",
    "            // m\n",
    "            if(d3.event.keyCode === 77) {\n",
    "                this.circle.call(this.force.drag);\n",
    "                this.svg.classed('m', true);\n",
    "            }\n",
    "\n",
    "            if(!this.selected_state && !this.selected_transition && !this.selected_frame) return;\n",
    "            switch(d3.event.keyCode) {\n",
    "                case 8:\n",
    "                case 66: // key delete\n",
    "                    if(!this.overtext){\n",
    "                        if(this.selected_state) {\n",
    "                            var new_transitions = this.model.get('transitions').slice();\n",
    "                            this.states.splice(this.states.indexOf(this.selected_state), 1);\n",
    "                            this.spliceTransitionsForState(this.selected_state, new_transitions);\n",
    "                            this.model.set('transitions', new_transitions)\n",
    "                            this.touch();\n",
    "                        }\n",
    "                        else if(this.selected_transition && !this.overtext) {\n",
    "                            if(this.selected_transition.target == this.selected_transition.source){\n",
    "                                var new_transitions = this.model.get('transitions').slice();\n",
    "                                new_transitions.splice(new_transitions.indexOf(this.selected_transition), 1);\n",
    "                                this.model.set('transitions', new_transitions)\n",
    "                                this.touch();\n",
    "                            }\n",
    "                            else {\n",
    "                                var new_transitions = this.model.get('transitions').slice();\n",
    "                                new_transitions.splice(new_transitions.indexOf(this.selected_transition), 1);\n",
    "                                this.model.set('transitions', new_transitions)\n",
    "                                this.touch();\n",
    "\n",
    "                            }\n",
    "                        }\n",
    "                        if(!this.overtext) {\n",
    "                            this.selected_transition = null;\n",
    "                            this.selected_state = null;\n",
    "                        }\n",
    "                        this.update();\n",
    "                        break;\n",
    "                    }\n",
    "                case 83: // key S : fix a node\n",
    "                    if(!this.overtext){\n",
    "                        if(this.selected_state) {\n",
    "                            this.selected_state.fixed = true;\n",
    "                        }\n",
    "                    }\n",
    "                    this.update();\n",
    "                    break;\n",
    "                case 70: // key F : create final transition\n",
    "                    if(!(this.isHidden(this.selected_state)) && !this.overtext){\n",
    "                        if(this.selected_state) {\n",
    "                            if(!(this.isFinal(this.selected_state, this.transitions))) {\n",
    "                                var pointx = this.selected_state.x,\n",
    "                                    pointy = this.selected_state.y,\n",
    "                                    state = {id: (this.selected_state.id) + 0.2};\n",
    "                                state.x = pointx +75;\n",
    "                                state.y = pointy;\n",
    "                                var new_states = this.model.get('states').slice();\n",
    "                                new_states.push(state);\n",
    "                                this.model.set('states', new_states);\n",
    "                                this.touch();\n",
    "                                var transition = {source: this.selected_state, target: state, label: ' '};\n",
    "                                var new_transitions = this.model.get('transitions').slice();\n",
    "                                new_transitions.push(transition);\n",
    "                                this.model.set('transitions', new_transitions);\n",
    "                                this.touch();\n",
    "                            }\n",
    "                            else { // delete final transitions\n",
    "                                var new_transitions = this.model.get('transitions').slice();\n",
    "                                this.spliceTransitionsForState(this.findFinalState(this.selected_state, this.states), new_transitions);\n",
    "                                this.states.splice(this.states.indexOf(this.findFinalState(this.selected_state, this.states)), 1);\n",
    "                                this.model.set('transitions', new_transitions);\n",
    "                                this.touch();\n",
    "                            }\n",
    "                        }\n",
    "                    this.update();\n",
    "                    break;\n",
    "                    }\n",
    "                case 73: // key I : create initial transition\n",
    "                    if(!(this.isHidden(this.selected_state)) && !this.overtext){\n",
    "                        if(this.selected_state) {\n",
    "                            if(!(this.isInitial(this.selected_state, this.transitions))) {\n",
    "                                var pointx = this.selected_state.x,\n",
    "                                    pointy = this.selected_state.y,\n",
    "                                    state = {id: (this.selected_state.id) + 0.1};\n",
    "                                state.x = pointx -75;\n",
    "                                state.y = pointy;\n",
    "                                var new_states = this.model.get('states').slice();\n",
    "                                new_states.push(state);\n",
    "                                this.model.set('states', new_states);\n",
    "                                this.touch();\n",
    "                                var transition = {source: state, target: this.selected_state, label: ' '};\n",
    "                                var new_transitions = this.model.get('transitions').slice();\n",
    "                                new_transitions.push(transition);\n",
    "                                this.model.set('transitions', new_transitions);\n",
    "                                this.touch();\n",
    "                            }\n",
    "                            else { // delete initial transitions\n",
    "                                var new_transitions = this.model.get('transitions').slice();\n",
    "                                this.spliceTransitionsForState(this.findInitialState(this.selected_state, this.states), new_transitions);\n",
    "                                this.states.splice(this.states.indexOf(this.findInitialState(this.selected_state, this.states)), 1);\n",
    "                                this.model.set('transitions', new_transitions);\n",
    "                                this.touch();\n",
    "                            }\n",
    "                        }\n",
    "                    this.update();\n",
    "                    break;\n",
    "                    }\n",
    "                case 76: // key L : create loop on node\n",
    "                    if(!(this.isHidden(this.selected_state)) && !this.overtext){\n",
    "                        if(this.selected_state && !(this.isLoop(this.selected_state, this.transitions))) {\n",
    "                            var loop = {source: this.selected_state, target: this.selected_state, label:'a', short: false};\n",
    "                            var new_transitions = this.model.get('transitions').slice();\n",
    "                            new_transitions.push(loop);\n",
    "                            this.model.set('transitions', new_transitions);\n",
    "                            this.touch();\n",
    "                        }\n",
    "                    this.update();\n",
    "                    break;\n",
    "                    }\n",
    "            }\n",
    "        },\n",
    "\n",
    "        keyup: function(doc, d, i){\n",
    "              this.lastKeyDown = -1;\n",
    "              // m\n",
    "              if(d3.event.keyCode === 77) {\n",
    "                this.circle\n",
    "                  .on('mousedown.drag', null)\n",
    "                  .on('touchstart.drag', null);\n",
    "                this.svg.classed('m', false);\n",
    "              }\n",
    "        },\n",
    "\n",
    "        // METHODS\n",
    "        idToState: function(id){\n",
    "            for(var j=0; j <= this.states.length -1; j++)\n",
    "                {\n",
    "                    if(this.states[j].id == id)\n",
    "                        {\n",
    "                            return this.states[j];\n",
    "                        }\n",
    "                }\n",
    "        },\n",
    "        existingTrans: function(transition, transitions){\n",
    "            for(var i=0; i<= transitions.length -1; i++){\n",
    "                if((transitions[i].source == transition.target) &&\n",
    "                  (transitions[i].target == transition.source)){\n",
    "                    return true;\n",
    "                    }\n",
    "                }\n",
    "            return false;\n",
    "        },\n",
    "        lastStateValidId: function(states){\n",
    "            for(var t = states.length -1; t>=0; t--)\n",
    "                    {\n",
    "                        if((parseFloat((states[t].id)) == parseInt((states[t].id)))\n",
    "                           && !isNaN((states[t].id)))\n",
    "                            {\n",
    "                                return states[t].id;\n",
    "                            }\n",
    "                    }\n",
    "\n",
    "        },\n",
    "        findFinalState: function(state, states){\n",
    "            for(var i = 0; i <= states.length -1; i++)\n",
    "                {\n",
    "                    if(states[i].id.toString() == (state.id.toString() + '.2'))\n",
    "                        {\n",
    "                            return states[i];\n",
    "                        }\n",
    "                }\n",
    "        },\n",
    "        findInitialState: function(state, states){\n",
    "            for(var i = 0; i <= states.length -1; i++)\n",
    "                {\n",
    "                    if(states[i].id.toString() == (state.id.toString() + '.1'))\n",
    "                        {\n",
    "                            return states[i];\n",
    "                        }\n",
    "                }\n",
    "        },\n",
    "        isInitial: function(state, transitions){\n",
    "                for(var i=0; i<= transitions.length -1; i++)\n",
    "                    {\n",
    "                        if(transitions[i].source.id == state.id ||\n",
    "                           transitions[i].target.id == state.id)\n",
    "                            {\n",
    "                                if(!((parseFloat((transitions[i].source.id)) ==\n",
    "                                      parseInt((transitions[i].source.id))) &&\n",
    "                                     !isNaN((transitions[i].source.id))) )\n",
    "                                    {\n",
    "                                        return true;\n",
    "                                    }\n",
    "                            }\n",
    "                    }\n",
    "                return false;\n",
    "        },\n",
    "        isFinal: function(state, transitions){\n",
    "                for(var i=0; i<= transitions.length -1; i++)\n",
    "                    {\n",
    "                        if(transitions[i].source.id == state.id ||\n",
    "                           transitions[i].target.id == state.id)\n",
    "                            {\n",
    "                                if(!((parseFloat((transitions[i].target.id)) ==\n",
    "                                      parseInt((transitions[i].target.id))) &&\n",
    "                                     !isNaN((transitions[i].target.id))) )\n",
    "                                    {\n",
    "                                        return true;\n",
    "                                    }\n",
    "                            }\n",
    "                    }\n",
    "                return false;\n",
    "        },\n",
    "        isHidden: function(state){\n",
    "                if(!(parseFloat(state.id) == parseInt(state.id) && !isNaN(state.id)))\n",
    "                    {\n",
    "                        return true;\n",
    "                    }\n",
    "                else{ return false;}\n",
    "        },\n",
    "        isShort: function(transition){\n",
    "                if(!((parseFloat((transition.target.id)) ==\n",
    "                      parseInt((transition.target.id))) &&\n",
    "                     !isNaN((transition.target.id)))\n",
    "                  || !((parseFloat((transition.source.id)) ==\n",
    "                        parseInt((transition.source.id))) &&\n",
    "                       !isNaN((transition.source.id))))\n",
    "                    {\n",
    "                        return true;\n",
    "                    }\n",
    "                else {return false;}\n",
    "\n",
    "        },\n",
    "        isLoop: function(state, transitions){\n",
    "                for(var i =0; i <= transitions.length -1; i++)\n",
    "                    {\n",
    "                        if(transitions[i].target.id == transitions[i].source.id)\n",
    "                            {\n",
    "                                if(transitions[i].target.id == state.id)\n",
    "                                    {\n",
    "                                        return true;\n",
    "                                    }\n",
    "                            }\n",
    "                    }\n",
    "                return false;\n",
    "        },\n",
    "        idToNode: function(id){\n",
    "            for(var j=0; j <= this.states.length -1; j++)\n",
    "                {\n",
    "                    if(this.states[j].id == id)\n",
    "                        {\n",
    "                            return this.states[j];\n",
    "                        }\n",
    "                }\n",
    "        },\n",
    "        completeTransitions: function(transitions){\n",
    "                for(var i =0; i <= transitions.length -1; i++)\n",
    "                    {\n",
    "                        if(transitions[i].length != 3)\n",
    "                            {\n",
    "                                if(typeof(transitions[i].source) === 'undefined')\n",
    "                                    {\n",
    "                                        //(transitions[i].target)+'.1'\n",
    "                                        var state = {id:((transitions[i].target)+'.1') };\n",
    "\n",
    "                                        this.states.push(state);\n",
    "                                        var trans = {source: state.id.toString()\n",
    "                                                     , target:transitions[i].target,\n",
    "                                                     label:transitions[i].label};\n",
    "                                        transitions.splice(i,1);\n",
    "                                        transitions.push(trans);\n",
    "                                        i --;\n",
    "                                        this.model.set('transitions', transitions);\n",
    "                                        this.touch();\n",
    "                                        this.model.set('states', this.states);\n",
    "                                        this.touch();\n",
    "                                    }\n",
    "                                else if(typeof(transitions[i].target) === 'undefined')\n",
    "                                    {\n",
    "                                        //(transitions[i].source)+'.2'\n",
    "                                        var state = {id:((transitions[i].source)+'.2')};\n",
    "                                        this.states.push(state);\n",
    "                                        var trans = {source: transitions[i].source,\n",
    "                                                     target: state.id.toString(),\n",
    "                                                     label:transitions[i].label}\n",
    "                                        transitions.splice(i,1);\n",
    "                                        transitions.push(trans);\n",
    "                                        i --;\n",
    "                                        this.model.set('transitions', transitions);\n",
    "                                        this.touch();\n",
    "                                        this.model.set('states', this.states);\n",
    "                                        this.touch();\n",
    "                                    }\n",
    "                            }\n",
    "                    }\n",
    "            return transitions;\n",
    "        },\n",
    "\n",
    "        // UPDATE THE VIEW\n",
    "        update: function(){\n",
    "\n",
    "            var colors = d3.scale.category10();\n",
    "            // Update force\n",
    "            this.force.nodes(this.states)\n",
    "                .links(this.transitions)\n",
    "\n",
    "            // Closure\n",
    "            var that = this;\n",
    "\n",
    "            // Update the datas on model\n",
    "            this.states = this.model.get('states');\n",
    "            this.transitions = this.model.get('transitions');\n",
    "            this.lastStateId = this.lastStateValidId(this.states);\n",
    "            this.model.set(\"lastStateId\", this.lastStateId);\n",
    "            this.model.set('states', this.states);\n",
    "            this.touch();\n",
    "\n",
    "            // Update the frame for label\n",
    "            this.frame.selectAll('input').attr('value', function(){\n",
    "                    if(that.selected_transition)\n",
    "                        this.value = that.selected_transition.label;\n",
    "                    else\n",
    "                        this.value = '';\n",
    "            })\n",
    "            this.frame.attr('value', function(){\n",
    "                    if(that.selected_transition)\n",
    "                        this.value = that.selected_transition.label;\n",
    "                    else\n",
    "                        this.value = '';\n",
    "            })\n",
    "                .on(\"mousedown\", function() {\n",
    "                    this.focus();\n",
    "                    that.overtext = true;\n",
    "                    that.selected_frame = true;\n",
    "                    that.update();\n",
    "                    if(that.selected_transition)\n",
    "                        this.value = that.selected_transition.label;\n",
    "                    else\n",
    "                        this.value = '';\n",
    "                })\n",
    "                .on(\"keypress\", function() {\n",
    "\n",
    "                    // IE fix\n",
    "                    if (!d3.event)\n",
    "                        d3.event = window.event;\n",
    "\n",
    "                    var e = d3.event;\n",
    "                    if (e.keyCode == 13)\n",
    "                    {\n",
    "                        if (typeof(e.cancelBubble) !== 'undefined') // IE\n",
    "                            e.cancelBubble = true;\n",
    "                        if (e.stopPropagation)\n",
    "                            e.stopPropagation();\n",
    "                        e.preventDefault();\n",
    "\n",
    "                        var txt = that.frame.node().value;\n",
    "\n",
    "                        that.selected_transition.label = txt;\n",
    "\n",
    "                        // trick to update automatically the label for vcsn\n",
    "                        var old_transitions = that.transitions;\n",
    "                        var last_transition = that.transitions[that.transitions.length -1]\n",
    "                        var new_transitions1 = that.model.get('transitions').slice();\n",
    "                        new_transitions1.splice(that.transitions.indexOf(that.transitions[that.transitions.length -1]),1)\n",
    "                        that.model.set('transitions', new_transitions1)\n",
    "                        var new_transitions2 = that.model.get('transitions').slice();\n",
    "                        new_transitions2.push(last_transition)\n",
    "                        that.model.set('transitions', new_transitions2)\n",
    "                        that.touch()\n",
    "\n",
    "                        that.model.set('transitions', old_transitions);\n",
    "                        that.touch();\n",
    "                        that.selected_transition = null;\n",
    "                        this.value = ''\n",
    "                        that.update();\n",
    "                    }\n",
    "                })\n",
    "\n",
    "            // Define elements groups and bound them data\n",
    "            this.path = this.path.data(this.transitions, function(d) {\n",
    "                return [d.source.id, d.target.id];});\n",
    "            this.circle = this.circle.data(this.states, function(d) {\n",
    "                return d.id;});\n",
    "\n",
    "            // TRANSITIONS\n",
    "            // Update existing transition\n",
    "            this.path.selectAll('path')\n",
    "                .classed('selected', function(d) {return d == that.selected_transition;})\n",
    "                .style('marker-end', function(d) {return 'url(#end-arrow)'; })\n",
    "                .style('stroke-dasharray', function(d) {\n",
    "                    if (d == that.selected_transition) {return \"10,2\";}\n",
    "                    else {return'';}})\n",
    "            // Enter new transition\n",
    "            var p = this.path.enter().append('svg:g');\n",
    "            p.append('svg:path')\n",
    "                .attr('class', 'transition')\n",
    "                .style('fill', 'none')\n",
    "                .style('stroke', '#000')\n",
    "                .style('stroke-width', '1.75px')\n",
    "                .style('cursor', 'pointer')\n",
    "                .classed('selected', function(d) { return d === that.selected_transition; })\n",
    "                .style('marker-end', function(d) {return 'url(#end-arrow)'; })\n",
    "                .style('stroke-dasharray', function(d) {\n",
    "                    if (d == that.selected_transition) {return \"10,2\";}\n",
    "                    else {return'';}})\n",
    "                .on('mousedown', function(d) {\n",
    "                      if(d3.event.keyCode === 77) return;\n",
    "                      // select transition\n",
    "                      that.mousedown_transition = d;\n",
    "                      if(that.mousedown_transition === that.selected_transition)\n",
    "                          that.selected_transition = null;\n",
    "                      else that.selected_transition = that.mousedown_transition;\n",
    "                      that.selected_state = null;\n",
    "                      that.selected_frame = false;\n",
    "                      that.update();\n",
    "                })\n",
    "\n",
    "            // Exit old transitions\n",
    "            this.path.exit().remove();\n",
    "\n",
    "            // TRANSITIONS LABELS\n",
    "            // Update existing labels\n",
    "            this.path.selectAll('text')\n",
    "                .text(function(d){return d.label});\n",
    "            // Show transition labels\n",
    "            p.append('svg:text')\n",
    "            .attr(\"class\", \"transitionlabelholder\")\n",
    "            .style(\"font-size\", \"18px\")\n",
    "            //.style(\"fill\", \"#00CCFF\")\n",
    "            .text(function(d) {return d.label});\n",
    "\n",
    "            // STATES\n",
    "            // Update existing states\n",
    "            this.circle.selectAll('circle')\n",
    "                .style('fill', function(d) {\n",
    "                    if (that.isHidden(d)) {return \"transparent\";}\n",
    "                    else {return (d === that.selected_state) ?\n",
    "                          d3.rgb(colors(d.id)).brighter().toString()\n",
    "                          : colors(d.id); }})\n",
    "                .style('stroke', function(d) {\n",
    "                    if (that.isHidden(d)) {return \"transparent\";}\n",
    "                    else {return d3.rgb(colors(d.id)).darker().toString(); }})\n",
    "            // Enter new states\n",
    "            var g = this.circle.enter().append('svg:g')\n",
    "            g.append('svg:circle')\n",
    "                .attr('class', 'state')\n",
    "                .attr('r', 20)\n",
    "                .style('stroke-width', '1.5px')\n",
    "                .style('cursor', 'pointer')\n",
    "                .style('fill', function(d) {\n",
    "                    if (that.isHidden(d)) {return \"transparent\";}\n",
    "                    else {return (d === that.selected_state) ?\n",
    "                          d3.rgb(colors(d.id)).brighter().toString()\n",
    "                          : colors(d.id); }})\n",
    "                .style('stroke', function(d) {\n",
    "                    if (that.isHidden(d)) {return \"transparent\";}\n",
    "                    else {return d3.rgb(colors(d.id)).darker().toString(); }})\n",
    "                .on('mouseover', function(d) {\n",
    "                    that.mouseover_state = d;\n",
    "                    if(that.isHidden(d)) {\n",
    "                        d3.select(this).style('stroke','black')\n",
    "                    }\n",
    "                })\n",
    "                .on('mouseout', function(d) {\n",
    "                    if(that.isHidden(d)) {\n",
    "                        d3.select(this).style('stroke','transparent')\n",
    "                    }\n",
    "                })\n",
    "                .on('mousedown', function(d) {\n",
    "                    d3.event.preventDefault();\n",
    "                    if(d3.event.keyCode === 77) return;\n",
    "                    // select state\n",
    "                    that.mousedown_state = d;\n",
    "                    if(that.mousedown_state === that.selected_state)\n",
    "                        that.selected_state = null;\n",
    "                    else {that.selected_state = that.mousedown_state;}\n",
    "                    that.selected_transition = null;\n",
    "                    // reposition drag line\n",
    "                    if(!that.isHidden(that.mousedown_state)  &&\n",
    "                       that.svg.classed('m') == false)\n",
    "                    {\n",
    "                        that.drag_line\n",
    "                            .style('marker-end', 'url(#end-arrow)')\n",
    "                            .classed('hidden', false)\n",
    "                            .attr('d', 'M' + that.mousedown_state.x + ',' +\n",
    "                                  that.mousedown_state.y + 'L' +\n",
    "                                  that.mousedown_state.x + ',' +\n",
    "                                  that.mousedown_state.y);\n",
    "                    }\n",
    "                    that.selected_frame = false;\n",
    "                    that.update();\n",
    "                })\n",
    "\n",
    "            // Exit old states\n",
    "            this.circle.exit().remove();\n",
    "\n",
    "            // STATES ID\n",
    "            // Update states ID\n",
    "            this.circle.selectAll('text')\n",
    "                .text(function(d) {\n",
    "                    if(that.isHidden(d))\n",
    "                    {return '';}\n",
    "                    else\n",
    "                    { return d.id;}\n",
    "                })\n",
    "\n",
    "            // Show states ID\n",
    "            g.append('svg:text')\n",
    "                .attr('x', 0)\n",
    "                .attr('y', 4)\n",
    "                .attr('class', 'id')\n",
    "            .style('font', '15px sans-serif')\n",
    "                .style(\"text-anchor\", \"middle\")\n",
    "                .style(\"fill\", function(d) {\n",
    "                    if(that.isHidden(d)) return \"transparent\";\n",
    "                    else return \"white\";})\n",
    "                .text(function(d) { return d.id; })\n",
    "                .style('cursor', 'pointer')\n",
    "                .on('mousedown', function(d) {\n",
    "                    d3.event.preventDefault();\n",
    "                    if(d3.event.keyCode === 77) return;\n",
    "                    // select state\n",
    "                    that.mousedown_state = d;\n",
    "                    if(that.mousedown_state === that.selected_state)\n",
    "                        that.selected_state = null;\n",
    "                    else {that.selected_state = that.mousedown_state;}\n",
    "                    that.selected_transition = null;\n",
    "                    // reposition drag line\n",
    "                    if(!that.isHidden(that.mousedown_state)  &&\n",
    "                       that.svg.classed('m') == false)\n",
    "                    {\n",
    "                        that.drag_line\n",
    "                            .style('marker-end', 'url(#end-arrow)')\n",
    "                            .classed('hidden', false)\n",
    "                            .attr('d', 'M' + that.mousedown_state.x + ',' +\n",
    "                                  that.mousedown_state.y + 'L' +\n",
    "                                  that.mousedown_state.x + ',' +\n",
    "                                  that.mousedown_state.y);\n",
    "                    }\n",
    "                    that.selected_frame = false;\n",
    "                    that.update();\n",
    "                })\n",
    "            this.force.start();\n",
    "        }\n",
    "    });\n",
    "\n",
    "    return {\n",
    "        AutomatonView: AutomatonView\n",
    "    }\n",
    "});"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 5,
   "metadata": {
    "collapsed": false
   },
   "outputs": [],
   "source": [
    "b = Automaton(states=[{'id':0}],transitions = [{'source':'0', 'label':''},{'target':'0','label':''}])\n",
    "display(b)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 6,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "[{'source': '0', 'label': ''}, {'target': '0', 'label': ''}]\n"
     ]
    }
   ],
   "source": [
    "print(get_ipython().user_ns['b'].transitions)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 7,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "[{'source': '0', 'label': ''}, {'label': '', 'target': '0'}]\n"
     ]
    },
    {
     "data": {
      "image/svg+xml": [
       "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>\n",
       "<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1//EN\"\n",
       " \"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd\">\n",
       "<!-- Generated by graphviz version 2.38.0 (20140413.2041)\n",
       " -->\n",
       "<!-- Title: %3 Pages: 1 -->\n",
       "<svg width=\"109pt\" height=\"37pt\"\n",
       " viewBox=\"0.00 0.00 109.00 37.00\" xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\">\n",
       "<g id=\"graph0\" class=\"graph\" transform=\"scale(1 1) rotate(0) translate(4 33)\">\n",
       "<title>%3</title>\n",
       "<polygon fill=\"white\" stroke=\"none\" points=\"-4,4 -4,-33 105,-33 105,4 -4,4\"/>\n",
       "<!-- I0 -->\n",
       "<g id=\"node1\" class=\"node\"><title>I0</title>\n",
       "<ellipse fill=\"black\" stroke=\"black\" cx=\"0\" cy=\"-14.5\" rx=\"0\" ry=\"0\"/>\n",
       "</g>\n",
       "<!-- 0 -->\n",
       "<g id=\"node3\" class=\"node\"><title>0</title>\n",
       "<ellipse fill=\"#98f5ff\" stroke=\"black\" cx=\"50.5\" cy=\"-14.5\" rx=\"14.5\" ry=\"14.5\"/>\n",
       "<text text-anchor=\"middle\" x=\"50.5\" y=\"-11.4\" font-family=\"Times,serif\" font-size=\"12.00\">0</text>\n",
       "</g>\n",
       "<!-- I0&#45;&gt;0 -->\n",
       "<g id=\"edge1\" class=\"edge\"><title>I0&#45;&gt;0</title>\n",
       "<path fill=\"none\" stroke=\"black\" d=\"M1.1391,-14.5C2.7053,-14.5 17.205,-14.5 29.922,-14.5\"/>\n",
       "<polygon fill=\"black\" stroke=\"black\" points=\"35.958,-14.5 29.958,-17.2001 32.958,-14.5 29.958,-14.5001 29.958,-14.5001 29.958,-14.5001 32.958,-14.5 29.958,-11.8001 35.958,-14.5 35.958,-14.5\"/>\n",
       "</g>\n",
       "<!-- F0 -->\n",
       "<g id=\"node2\" class=\"node\"><title>F0</title>\n",
       "<ellipse fill=\"black\" stroke=\"black\" cx=\"101\" cy=\"-14.5\" rx=\"0\" ry=\"0\"/>\n",
       "</g>\n",
       "<!-- 0&#45;&gt;F0 -->\n",
       "<g id=\"edge2\" class=\"edge\"><title>0&#45;&gt;F0</title>\n",
       "<path fill=\"none\" stroke=\"black\" d=\"M65.1067,-14.5C74.9272,-14.5 87.3767,-14.5 94.7355,-14.5\"/>\n",
       "<polygon fill=\"black\" stroke=\"black\" points=\"100.939,-14.5 94.9394,-17.2001 97.9394,-14.5 94.9394,-14.5001 94.9394,-14.5001 94.9394,-14.5001 97.9394,-14.5 94.9393,-11.8001 100.939,-14.5 100.939,-14.5\"/>\n",
       "</g>\n",
       "</g>\n",
       "</svg>\n"
      ],
      "text/plain": [
       "mutable_automaton<context<letterset<char_letters>, b>>"
      ]
     },
     "execution_count": 7,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "print(b.transitions)\n",
    "# From d3 to vcsn\n",
    "res = ''\n",
    "for t in b.transitions:\n",
    "    src = t['source'] if 'source' in t.keys() else '$'\n",
    "    dst = t['target'] if 'target' in t.keys() else '$'\n",
    "    res += \"{} -> {} {}\\n\".format(src, dst, t['label'])\n",
    "vcsn.automaton(res, 'daut')"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 8,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "0"
      ]
     },
     "execution_count": 8,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "# We can access to many thing on D3 model, here last State ID\n",
    "b.lastStateId"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 9,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "[{'label': '', 'source': '0'}, {'label': '', 'target': '0'}]"
      ]
     },
     "execution_count": 9,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "# We can access to transitions on D3 model\n",
    "b.transitions"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 10,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "[{'id': 0}]"
      ]
     },
     "execution_count": 10,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "# We can access to states on D3 model\n",
    "b.states"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 11,
   "metadata": {
    "collapsed": false
   },
   "outputs": [],
   "source": [
    "# Here if we add an other nodes, then the force layout isn't run at the same time\n",
    "# Have to check for Backbone.mode.changet and Widget.hold._sync\n",
    "# and may add a posx, posy for every nodes\n",
    "b.states = [{'id': 0}, {'id': 1}, {'id': 2}, {'id': 3}]"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 12,
   "metadata": {
    "collapsed": false
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "[{'label': '', 'source': '0'}, {'label': '', 'target': '0'}]"
      ]
     },
     "execution_count": 12,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "# Same problem as nodes\n",
    "b.transitions"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 13,
   "metadata": {
    "collapsed": false
   },
   "outputs": [],
   "source": [
    "a = Automaton(states = [{'id': 0, 'initial': False, 'final': False, 'hidden': False}],\n",
    "              transitions = [{'source': 0, 'target': 0, 'label': 'a', 'short': False}])\n",
    "display(a)"
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.4.4"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 0
}
